@@ -19,7 +19,7 @@ from Utils.MemoryCache import MemoryCache
 from Utils.Timers import timeFunction
 from WMCore.DAOFactory import DAOFactory
 from WMCore.Services.Rucio.Rucio import Rucio, WMRucioException
-from WMCore.Services.Rucio.RucioUtils import RUCIO_VALID_PROJECT
+from WMCore.Services.Rucio.RucioUtils import RUCIO_VALID_PROJECT, RUCIO_RULES_PRIORITY
 from WMCore.WMException import WMException
 from WMCore.WorkerThreads.BaseWorkerThread import BaseWorkerThread
 from Utils.IteratorTools import grouper
@@ -362,16 +362,16 @@ class RucioInjectorPoller(BaseWorkerThread):
 
         # Get list of blocks that can be deleted
         # blockDict = self.findDeletableBlocks.execute(transaction=False)
-        completedBlocksList = self.getCompletedBlocks.execute(transaction=False)
+        completedBlocksDict = self.getCompletedBlocks.execute(transaction=False)
 
-        if not completedBlocksList:
+        if not completedBlocksDict:
             logging.info("No candidate blocks found for rule deletion.")
             return
 
-        logging.info("Found %d completed blocks", len(completedBlocksList))
-        logging.debug("Full completedBlocksList: %s", pformat(completedBlocksList))
+        logging.info("Found %d completed blocks", len(completedBlocksDict))
+        logging.debug("Full completedBlocksDict: %s", pformat(completedBlocksDict))
 
-        deletableWfsDict = self.getDeletableWorkflows.execute()
+        deletableWfsDict = set(self.getDeletableWorkflows.execute())
 
         if not deletableWfsDict:
             logging.info("No workflow chains (Parent + child workflows) in fully terminal state found. Skipping block level rule deletion in the current run.")
@@ -382,8 +382,8 @@ class RucioInjectorPoller(BaseWorkerThread):
 
         now = int(time.time())
         blockDict = {}
-        for block in completedBlocksList:
-            if block['workflowName'] in deletableWfsDict and \
+        for block in completedBlocksDict.values():
+            if block['workflowNames'].issubset(deletableWfsDict) and \
                now - block['blockCreateTime'] > self.blockDeletionDelaySeconds:
                 blockDict[block['blockName']] = block
 
@@ -391,14 +391,15 @@ class RucioInjectorPoller(BaseWorkerThread):
         logging.debug("Final deletable blocks dict: %s", pformat(blockDict))
 
         for blocksSlice in grouper(blockDict, self.delBlockSlicesize):
-            logging.info("Handeling %d candidate blocks", len(blocksSlice))
+            logging.info("Handling %d candidate blocks", len(blocksSlice))
             containerDict = {}
-            # Populate containerDict, assigning each block to its correspondant container
+            # Populate containerDict, assigning each block to its correspondent container
             for blockName in blocksSlice:
                 container = blockDict[blockName]['dataset']
                 # If the container is not in the dictionary, create a new entry for it
                 if container not in containerDict:
-                    # Set of sites to which the container needs to be transferred
+                    # All blocks belonging to a container need to be sent to the same sites, so we simply take the sites list 
+                    # from the current block to determine the containers required final RSEs.
                     sites = set(x.replace("_MSS", "_Tape") for x in blockDict[blockName]['sites'])
                     containerDict[container] = {'blocks': [], 'rse': sites}
                 containerDict[container]['blocks'].append(blockName)
@@ -490,6 +491,7 @@ class RucioInjectorPoller(BaseWorkerThread):
             rseName = subInfo['site'].replace("_MSS", "_Tape")
             container = subInfo['path']
             lifetime = subInfo['dataset_lifetime']
+            rulepriority = RUCIO_RULES_PRIORITY.get(subInfo['priority'])
             # Skip central production Tape rules
             if not self.isT0agent and rseName.endswith("_Tape"):
                 logging.info("Bypassing Production container Tape data placement for container: %s and RSE: %s",
@@ -506,6 +508,7 @@ class RucioInjectorPoller(BaseWorkerThread):
                               account=self.rucioAcct,
                               grouping="ALL",
                               comment=ruleComment,
+                              priority=rulepriority,
                               meta=self.metaData)
             if not rseName.endswith(("_Tape", "_Tape_Test")):
                 # add extra parameters to the Disk rule as defined in the component configuration
@@ -518,7 +521,6 @@ class RucioInjectorPoller(BaseWorkerThread):
                     rseName = rseName.replace("cms_type=real", "cms_type=test")
             else:
                 # then it's a T0 container placement
-                ruleKwargs['priority'] = 4
                 if not rseName.endswith("_Tape") and lifetime > 0:
                     ruleKwargs['lifetime'] = lifetime
                 if self.testRSEs:
